/****************************************************************************
** Meta object code from reading C++ file 'kapmanmainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kapman-26.07.80/src/kapmanmainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kapmanmainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16KapmanMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto KapmanMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN16KapmanMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KapmanMainWindow",
        "initGame",
        "",
        "handleGameOver",
        "newGame",
        "showHighscores",
        "changeLevel",
        "setSoundsEnabled",
        "p_enabled",
        "showSettings",
        "onThemeChanged",
        "const KGameTheme*",
        "theme",
        "close",
        "displayLevel",
        "p_level",
        "displayScore",
        "p_score",
        "displayLives",
        "p_lives",
        "resetStatusBar",
        "viewFullScreen",
        "fullScreen"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'initGame'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleGameOver'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'newGame'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showHighscores'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changeLevel'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setSoundsEnabled'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'showSettings'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onThemeChanged'
        QtMocHelpers::SlotData<void(const KGameTheme *)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'displayLevel'
        QtMocHelpers::SlotData<void(unsigned int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Slot 'displayScore'
        QtMocHelpers::SlotData<void(unsigned int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::UInt, 17 },
        }}),
        // Slot 'displayLives'
        QtMocHelpers::SlotData<void(unsigned int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::UInt, 19 },
        }}),
        // Slot 'resetStatusBar'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewFullScreen'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KapmanMainWindow, qt_meta_tag_ZN16KapmanMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KapmanMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KapmanMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KapmanMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16KapmanMainWindowE_t>.metaTypes,
    nullptr
} };

void KapmanMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KapmanMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->initGame(); break;
        case 1: _t->handleGameOver(); break;
        case 2: _t->newGame(); break;
        case 3: _t->showHighscores(); break;
        case 4: _t->changeLevel(); break;
        case 5: _t->setSoundsEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->showSettings(); break;
        case 7: _t->onThemeChanged((*reinterpret_cast<std::add_pointer_t<const KGameTheme*>>(_a[1]))); break;
        case 8: _t->close(); break;
        case 9: _t->displayLevel((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 10: _t->displayScore((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 11: _t->displayLives((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 12: _t->resetStatusBar(); break;
        case 13: _t->viewFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KapmanMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KapmanMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KapmanMainWindowE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KapmanMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
