// This file is generated by kconfig_compiler_kf6 from kapman.kcfg.
// All changes you do to this file will be lost.
#ifndef SETTINGS_H
#define SETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class Settings : public KConfigSkeleton
{
  public:

    static Settings *self();
    ~Settings() override;

    /**
      Set The graphical theme to be used.
    */
    static
    void setTheme( const QString & v )
    {
      if (!self()->isThemeImmutable())
        self()->mTheme = v;
    }

    /**
      Get The graphical theme to be used.
    */
    static
    QString theme()
    {
      return self()->mTheme;
    }

    /**
      Is The graphical theme to be used. Immutable
    */
    static
    bool isThemeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Theme" ) );
    }

    /**
      Set Whether sound effects should be played.
    */
    static
    void setSounds( bool v )
    {
      if (!self()->isSoundsImmutable())
        self()->mSounds = v;
    }

    /**
      Get Whether sound effects should be played.
    */
    static
    bool sounds()
    {
      return self()->mSounds;
    }

    /**
      Is Whether sound effects should be played. Immutable
    */
    static
    bool isSoundsImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Sounds" ) );
    }

  protected:
    Settings();
    friend class SettingsHelper;


    // General
    QString mTheme;
    bool mSounds;
};

#endif

